
# instancer.py
# Example of how to use this script.
# 1. select a cube,
# 2. shift click to select a polymesh
# 3. in the script editor (python tab) enter these commands,
#
# import instancer
# instancer.instancer()
# or
# instancer.instancer(False)
#
# A cube will be placed at each vertex of the polymesh.

import maya.cmds as cmds
from random import uniform

# Select the object that you wish to instance, such as a polyplane
# or RenderMan light. Hold down the shift key and select the poly
# mesh object that you wish to "cover" with instances.
def instancer(use_random_rotation=True):
	items = cmds.ls(sl=True)
	if len(items) < 2:
		cmds.error('Two items must be selected')
		return
	print('You have chosen to use "%s" for instancing.\n' % items[0])
	print('You have chosen the polymesh "%s" to receive the instances.\n' % items[1])

	# Use the first selection
	tnode = items[0]
	
	# Get the positions of the vertices of the second selection
	verts = get_verts(items[1], False)

	instances = []
	for x,y,z in verts:
		inst = cmds.instance(tnode)[0]
		if use_random_rotation:
			x_angle = uniform(0, 360)
			y_angle = uniform(0, 360)
			z_angle = uniform(0, 360)
			cmds.rotate(x_angle, y_angle, z_angle, inst, relative=True)
			 
		cmds.move(x,y,z,inst)
		instances.append(inst)
	cmds.group(instances)
	
# Returns a list of lists ie. [[x,y,z],[x,y,z]....] 
def get_verts(tnode, useLocalSpace=True):
	verts = []
	shape = cmds.listRelatives(tnode, shapes=True)[0]
	num_verts = cmds.polyEvaluate(tnode, vertex=True)
	for n in range(num_verts):
		vert_str = shape + '.vtx[%d]' % n;
		vert_pos = cmds.pointPosition(vert_str, local=useLocalSpace) 
		verts.append(vert_pos)	
	return verts
	
