import maya.cmds as cmds


def addFloat(attrname, value, min_max=[0,10.0]):
	selections = cmds.ls(sl=True)
	for sel in selections:
		shapes = cmds.listRelatives(sel, s=True)
		# sel is a shape, therefore, does not have shape relatives
		if shapes == None:
			shapes = [sel]
		if len(shapes) == 0 or len(shapes[0]) == 0:
			print('Cannot find a shape to add attribute')
			return
		shape = shapes[0]
		if cmds.attributeQuery(attrname, node=shape, exists=True) == False:
			cmds.addAttr(shape, ln=attrname, sn=attrname, k=True, at='double',
						min=min_max[0], max=min_max[1], dv=value)
		cmds.setAttr(shape + '.' + attrname, value)
	print('Added "%s" with a float value of %1.3f' % (attrname, float(value)))
	

def addInt(attrname, value, min_max=[0,10.0]):
	value = int(value)
	selections = cmds.ls(sl=True)
	for sel in selections:
		shapes = cmds.listRelatives(sel, s=True)
		# sel is a shape, therefore, does not have shape relatives
		if shapes == None:
			shapes = [sel]
		if len(shapes) == 0 or len(shapes[0]) == 0:
			print('Cannot find a shape to add attribute')
			return
		shape = shapes[0]
		if cmds.attributeQuery(attrname, node=shape, exists=True) == False:
			cmds.addAttr(shape, ln=attrname, sn=attrname, at='long',  
						min=min_max[0], max=min_max[1], dv=value)
		cmds.setAttr(shape + '.' + attrname, value)
	print('Added "%s" with a int value of %d' % (attrname, value))

def addString(attrname):
	selections = cmds.ls(sl=True)
	for sel in selections:
		shapes = cmds.listRelatives(sel, s=True)
		# sel is a shape, therefore, does not have shape relatives
		if shapes == None:
			shapes = [sel]
		if len(shapes) == 0 or len(shapes[0]) == 0:
			print('Cannot find a shape to add attribute')
			return
		shape = shapes[0]
		if cmds.attributeQuery(attrname, node=shape, exists=True) == False:
			cmds.addAttr(shape, ln=attrname, sn=attrname, dt='string')
		#cmds.setAttr(shape + '.' + attrname, value)
	print('Added string attr "%s"' % attrname)




