
# vector_utils.py        
# From Ronald Goldman is presented as this C function.
# http://paulbourke.net/geometry/rotate/

import math
import random

class Point3D(object):
	def __init__(self,x,y,z):
		self.x = float(x)
		self.y = float(y)
		self.z = float(z)
	# This next method should not be part of the Point3D
	# class because it is conceptually impossible to normalize
	# a x,y,z location in space!!
	def normalize(self):
		length = math.sqrt((self.x * self.x) + (self.y * self.y) + (self.z * self.z))
		self.x /= length
		self.y /= length
		self.z /= length
		
class Vector3D(Point3D):
	@staticmethod
	# NOTE: the circle is assumed to be on the XZ plane
	def getRandomVectorOnCircle(rad):
		#find a random point within a 2 * rad square
		x = random.uniform(-rad, rad)
		z = random.uniform(-rad, rad)
		vector3d = Vector3D(x, 0, z)
  		#convert to a unit vector - this will place
		#head of the vector on a circle of radius 1.
		#scale it by rad.
		vector3d.normalize()
		vector3d.scale(rad)
		return vector3d
	#----------------------------------------
	@staticmethod
	# Returns the x-rotation and z-rotation, measured in degrees, such
	# if they were applied to an object its Y-axis would be aligned to the
	# direction of the input vector3d.
	def aimY(vector3d):
		xy_length = math.sqrt(vector3d.x * vector3d.x + vector3d.y * vector3d.y)
		vec_length= vector3d.getLength()
	
		x_rot = math.acos(xy_length/vec_length)
		x_rot = x_rot if vector3d.z > 0 else -x_rot
		x_rot = math.degrees(x_rot)
			
		# xy_length will be zero when vec is pointing
		# along the +z or -z axis
		if xy_length == 0:
			z_rot = math.radians(90) if vector3d.x > 0 else math.radians(-90)
		else:
			z_rot = math.acos(vector3d.y/xy_length)
		z_rot = -z_rot if vector3d.x > 0 else z_rot
		z_rot = math.degrees(z_rot)
		return [x_rot, z_rot]
	
	def __init__(self,x,y,z):
		Point3D.__init__(self,x,y,z)
	def scale(self, sc):
		self.x *= sc
		self.y *= sc
		self.z *= sc
	def dot(self, vec3d):
		return self.x * vec3d.x + self.y * vec3d.y + self.z * vec3d.z
	def unit(self):
		self.normalize()
	def add(self, vec3d):
		self.x += vec3d.x
		self.y += vec3d.y
		self.z += vec3d.z
	def subtract(self, vec3d):
		self.x -= vec3d.x
		self.y -= vec3d.y
		self.z -= vec3d.z
	def getLength(self):
		return math.sqrt(self.x * self.x + self.y * self.y + self.z * self.z)
		
	def rotateAroundAxis(self, angle, axis3d):
		axis3d.normalize()
		costheta = math.cos(math.radians(angle));
		sintheta = math.sin(math.radians(angle));
		
		X = (costheta + (1 - costheta) * axis3d.x * axis3d.x) * self.x
		X += ((1 - costheta) * axis3d.x * axis3d.y - axis3d.z * sintheta) * self.y
		X += ((1 - costheta) * axis3d.x * axis3d.z + axis3d.y * sintheta) * self.z
		#-------------------------------------------------------------------------------
		Y = ((1 - costheta) * axis3d.x * axis3d.y + axis3d.z * sintheta) * self.x
		Y += (costheta + (1 - costheta) * axis3d.y * axis3d.y) * self.y
		Y += ((1 - costheta) * axis3d.y * axis3d.z - axis3d.x * sintheta) * self.z
		#-------------------------------------------------------------------------------
		Z = ((1 - costheta) * axis3d.x * axis3d.z - axis3d.y * sintheta) * self.x
		Z += ((1 - costheta) * axis3d.y * axis3d.z + axis3d.x * sintheta) * self.y
		Z += (costheta + (1 - costheta) * axis3d.z * axis3d.z) * self.z
		self.x = X
		self.y = Y
		self.z = Z
# Note: the next function is not used by the tree.
"""Rotate a point p by angle theta around an arbitrary line segment p1-p2
   Return the rotated point.
   Positive angles are anticlockwise looking down the axis
   towards the origin.
   Assume right hand coordinate system.  """
def arbitrary_rotate2(point3d, angle, axis3d, point3d1, point3d2):
	q = Point3D(0,0,0)
	"""
	r.x = p2.x - p1.x;
	r.y = p2.y - p1.y;
	r.z = p2.z - p1.z;
	p.x -= p1.x;
	p.y -= p1.y;
	p.z -= p1.z;
	"""
	axis3d.x = point3d2.x - point3d1.x
	axis3d.y = point3d2.y - point3d1.y
	axis3d.z = point3d2.z - point3d1.z
	point3d.x = point3d1.x
	point3d.y = point3d1.y
	point3d.z = point3d1.z
	#-------------------------------------------------------------------------------
	axis3d.normalize()
	
	costheta = math.cos(math.radians(angle));
	sintheta = math.sin(math.radians(angle));
	#-------------------------------------------------------------------------------
	"""
	q.x += (costheta + (1 - costheta) * r.x * r.x) * p.x;
	q.x += ((1 - costheta) * r.x * r.y - r.z * sintheta) * p.y;
	q.x += ((1 - costheta) * r.x * r.z + r.y * sintheta) * p.z;
	"""
	q.x += (costheta + (1 - costheta) * axis3d.x * axis3d.x) * point3d.x
	q.x += ((1 - costheta) * axis3d.x * axis3d.y - axis3d.z * sintheta) * point3d.y;
	q.x += ((1 - costheta) * axis3d.x * axis3d.z + axis3d.y * sintheta) * point3d.z;
	#-------------------------------------------------------------------------------
	"""
	q.y += ((1 - costheta) * r.x * r.y + r.z * sintheta) * p.x;
	q.y += (costheta + (1 - costheta) * r.y * r.y) * p.y;
	q.y += ((1 - costheta) * r.y * r.z - r.x * sintheta) * p.z;
	"""
	q.y += ((1 - costheta) * axis3d.x * axis3d.y - axis3d.z * sintheta) * point3d.x;
	q.y += (costheta + (1 - costheta) * axis3d.y * axis3d.y) * point3d.y
	q.y += ((1 - costheta) * axis3d.y * axis3d.z + axis3d.x * sintheta) * point3d.z;
	#-------------------------------------------------------------------------------
	"""
	q.z += ((1 - costheta) * r.x * r.z - r.y * sintheta) * p.x;
	q.z += ((1 - costheta) * r.y * r.z + r.x * sintheta) * p.y;
	q.z += (costheta + (1 - costheta) * r.z * r.z) * p.z;
	"""
	q.z += ((1 - costheta) * axis3d.x * axis3d.z - axis3d.y * sintheta) * point3d.x;
	q.z += ((1 - costheta) * axis3d.y * axis3d.z + axis3d.x * sintheta) * point3d.y;
	q.z += (costheta + (1 - costheta) * axis3d.z * axis3d.z) * point3d.z
	"""
	q.x += p1.x;
	q.y += p1.y;
	q.z += p1.z;
	"""
	q.x += point3d1.x
	q.y += point3d1.y
	q.z += point3d1.z
	
	return q
	

	
	
if __name__ == '__main__':
	point3d = Point3D(2,3,5)
	
	p1 = Point3D(1,1,1)
	p2 = Point3D(5,10,3)
	angle = 77
	axis3d = Vector3D(0,1,0)
	
	ar = arbitrary_rotate(point3d, angle, axis3d)
	ar2 = arbitrary_rotate2(point3d, angle, axis3d,p1, p2)
	print("X: %f  Y: %f  Z: %f " % (ar.x, ar.y, ar. z))
	print("X: %f  Y: %f  Z: %f " % (ar2.x, ar2.y, ar2. z))
	
	vec3d = Vector3D(1,1,0)
	axis3d = Vector3D(0,1,0)
	vec3d.reflect(axis3d)
	print("refect X: %f  Y: %f  Z: %f " % (vec3d.x, vec3d.y, vec3d. z))
	
	point3d = Vector3D(1,1,0)
	axis3d = Vector3D(0,1,0)
	point3d.rotateAroundAxis(180, axis3d)
	print("X: %f  Y: %f  Z: %f " % (point3d.x, point3d.y, point3d. z))	
	
	point3d = Point3D(1,1,0)
	pnt = arbitrary_rotate(point3d, 180, axis3d)
	print("X: %f  Y: %f  Z: %f " % (pnt.x, pnt.y, pnt. z))	
	
	print( Vector3D.aimY(Vector3D(1,1,0)) )
	
	
